LOG_LEVEL_ENV = "LOG_LEVEL"
DEFAULT_LOG_LEVEL = "INFO"

ENTRYPOINT_ENV = "ENTRYPOINT"
DEFAULT_ENTRYPOINT = "user_code.src.program:run"

INPUT_DIRECTORY_ENV = "INPUT_DIRECTORY"
DEFAULT_INPUT_DIRECTORY = "/var/runtime/input"

OUTPUT_DIRECTORY_ENV = "OUTPUT_DIRECTORY"
DEFAULT_OUTPUT_DIRECTORY = "/var/runtime/output"

WORKSPACE_ENV = "WORKSPACE"
DEFAULT_WORKSPACE = "./user_code"

DATAPOOL_DIRECTORY_ENV = "DATAPOOL_DIRECTORY"
DEFAULT_DATAPOOL_DIRECTORY = "/var/runtime/datapool"
